"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var chalk_1 = require("chalk");
var Table = require("cli-table3");
var log = require("fancy-log");
var gzipSize = require("gzip-size");
var path = require("path");
var PluginError = require("plugin-error");
var prettyBytes = require("pretty-bytes");
var through = require("through2");
var internal_1 = require("../internal");
exports.sizer = function (options) {
    options = options || {};
    options.gzip = options.gzip !== undefined ? options.gzip : false;
    options.minifier = options.minifier !== undefined ? options.minifier : null;
    options.total = options.total !== undefined ? options.total : true;
    options.fail = options.fail !== undefined ? options.fail : false;
    var tableHeadCols = ['File', 'Original'];
    var fail = false;
    if (options.gzip) {
        tableHeadCols.push('Gzipped');
    }
    if (options.minifier) {
        tableHeadCols.push('Minified');
        if (options.gzip) {
            tableHeadCols.push('Gzipped');
        }
    }
    var fileCount = 0;
    var totalSize = 0;
    var totalGzippedSize = 0;
    var totalMinifiedSize = 0;
    var totalMinifiedGzippedSize = 0;
    var gzippedSize;
    var minifiedGzippedSize;
    var minified;
    var table = new Table({
        chars: {
            bottom: '═',
            'bottom-left': '╚',
            'bottom-mid': '╧',
            'bottom-right': '╝',
            left: '║',
            'left-mid': '╟',
            mid: '─',
            'mid-mid': '┼',
            middle: '│',
            right: '║',
            'right-mid': '╢',
            top: '═',
            'top-left': '╔',
            'top-mid': '╤',
            'top-right': '╗',
        },
        colAligns: ['left', 'right', 'right', 'right', 'right'],
        head: tableHeadCols,
        // colWidths: [ 30, 12, 12, 12, 12 ],
        style: {
            head: ['cyan'],
        },
    });
    var getSizeToDisplay = function (size, key, filename) {
        var max = options[filename] || options['*'];
        var value;
        value = max ? max[key] : options[key];
        if (value && size > value) {
            fail = true;
            return chalk_1.default.red(prettyBytes(size));
        }
        return prettyBytes(size);
    };
    return through.obj(function (file, enc, callback) {
        if (file.isNull()) {
            callback(null, file);
            return;
        }
        if (file.isStream()) {
            callback(new PluginError('gulp-sizereport', 'Streaming not supported'));
            return;
        }
        gzippedSize = gzipSize.sync(file.contents);
        totalSize += file.contents.length;
        totalGzippedSize += gzippedSize;
        fileCount++;
        var row = [
            path.join(options.dest, file.relative),
            getSizeToDisplay(file.contents.length, 'maxSize', file.relative),
        ];
        if (options.gzip) {
            row.push(getSizeToDisplay(gzippedSize, 'maxGzippedSize', file.relative));
        }
        if (typeof options.minifier === 'function') {
            minified = options.minifier('' + file.contents, file.relative);
            totalMinifiedSize += minified.length;
            row.push(getSizeToDisplay(minified.length, 'maxMinifiedSize', file.relative));
            if (options.gzip) {
                minifiedGzippedSize = gzipSize.sync(minified);
                totalMinifiedGzippedSize += minifiedGzippedSize;
                row.push(getSizeToDisplay(minifiedGzippedSize, 'maxMinifiedGzippedSize', file.relative));
            }
        }
        table.push(row);
        callback(null, file);
    }, function (callback) {
        if (options.title) {
            // eslint-disable-next-line
            log(':: ' + chalk_1.default.bold(options.title) + ' ::');
        }
        if (fileCount > 0) {
            if (options.total === true) {
                var row = ['', chalk_1.default.bold(getSizeToDisplay(totalSize, 'maxTotalSize', '*'))];
                if (options.gzip) {
                    row.push(chalk_1.default.bold(getSizeToDisplay(totalGzippedSize, 'maxTotalGzippedSize', '*')));
                }
                if (options.minifier) {
                    row.push(chalk_1.default.bold(getSizeToDisplay(totalMinifiedSize, 'maxTotalMinifiedSize', '*')));
                    if (options.gzip) {
                        row.push(chalk_1.default.bold(getSizeToDisplay(totalMinifiedGzippedSize, 'maxTotalMinifiedGzippedSize', '*')));
                    }
                }
                table.push(row);
            }
            internal_1.logTitle('Size Report');
            console.log(table.toString());
            if (options.fail && fail) {
                callback();
            }
        }
        callback();
    });
};
//# sourceMappingURL=sizer.js.map