import * as autoprefixer from 'autoprefixer';
import { TransformOptions } from 'babel-core';
import browserSync = require('browser-sync');
import { BrowserSyncInstance } from 'browser-sync';
import { CssNanoOptions } from 'cssnano';
import { Options } from 'del';
import { CLIEngine } from 'eslint';
import { Gulp, TaskFunction } from 'gulp';
import * as postcss from 'postcss';
import { DestOptions, SrcOptions } from 'vinyl-fs';
import * as webpack from 'webpack';
import { BundleAnalyzerPlugin } from 'webpack-bundle-analyzer';
import { WebpackMerge as IWebpackMerge } from 'webpack-merge';
import { ITimplaProcess } from '../internal';
export declare type IRecursivePartial<T> = {
    [P in keyof T]?: IRecursivePartial<T[P]>;
};
export declare type IPartialExcept<T, K extends keyof T> = IRecursivePartial<T> & Pick<T, K>;
export declare type ITimplaBrowserSyncOptions = browserSync.Options & {
    /** Current working directory. Sets the base path for file watchers etc. */
    cwd: string;
    /**
     * Toggle default BrowserSync middlewares added by Timpla.
     *
     * Set to `false` to disable all default middleware from being added. Err on the side of caution as this may break browserSync and JS from working altogether.
     *
     * When using proxy-mode, set disableSpa: `true` to ensure that all requests go through to your proxied server as normal.
     */
    middlewareConfig: false | {
        /** Disables webpackDevMiddleware. */
        disableDev: boolean;
        /** Disables webpackHotMiddleware. */
        disableHot: boolean;
        /** Disables historyApiFallbackMiddleware. */
        disableSpa: boolean;
    };
    /** Disables the `fallback server config` that Timpla uses when no browserSync config is supplied. */
    disableServerFallback: boolean;
};
export interface IOpenOptions {
    /** The target to open. */
    target: string;
    options?: {
        /**
         * Wait for the opened app to exit before fulfilling the promise.
         * If `false` it's fulfilled immediately when opening the app.
         * On Windows you have to explicitly specify an app for it to be able to wait.
         * Defaults to `true`.
         */
        wait?: boolean;
        /**
         * Specify the app to open the `target` with, or an array with the app and app arguments.
         * The app name is platform dependent. Don't hard code it in reusable modules.
         * Eg. Chrome is `google chrome` on OS X, `google-chrome` on Linux and `chrome` on Windows.
         */
        app?: string | ReadonlyArray<string>;
    };
}
export declare type INodeSassImporterReturnType = {
    file: string;
} | {
    contents: string;
} | Error | null;
export declare type INodeSassImporter = (url: string, prev: string, done: (data: INodeSassImporterReturnType) => void) => INodeSassImporterReturnType | void;
export interface INodeSassOptions {
    file?: string;
    data?: string;
    importer?: INodeSassImporter | INodeSassImporter[];
    functions?: {
        [key: string]: () => any;
    };
    includePaths?: string[];
    indentedSyntax?: boolean;
    indentType?: string;
    indentWidth?: number;
    linefeed?: string;
    omitSourceMapUrl?: boolean;
    outFile?: string;
    outputStyle?: 'compact' | 'compressed' | 'expanded' | 'nested';
    precision?: number;
    sourceComments?: boolean;
    sourceMap?: boolean | string;
    sourceMapContents?: boolean;
    sourceMapEmbed?: boolean;
    sourceMapRoot?: string;
}
export declare type ITimplaCustomizeWebpackConfig = (timplaWebpackHelper: ITimplaWebpackHelper) => webpack.Configuration;
export declare type ITimplaWebpackResolver = (...args: string[]) => string[];
/**
 * Timpla supports the webpack object entry format.
 */
export interface IWebpackObjectEntry {
    [name: string]: string[] | string;
}
/**
 * A function that resolves entries
 * to relative their relative paths.
 */
export declare type ITimplaWebpackEntryHook = ({ resolve, }: {
    resolve: ITimplaWebpackResolver;
}) => IWebpackObjectEntry;
export declare type ITimplaWebpackEntry = ITimplaWebpackEntryHook | IWebpackObjectEntry;
export interface IGulpSvgStore {
    /**
     *  Removes the wrapping <xml> node and DOC export type for inline-use, defaults to `false`.
     */
    inlineSvg?: boolean;
}
export declare type ITimplaTask = (timplaConfig: IFullTimplaConfig) => TaskFunction;
export declare type ITimplaHofTask = (timplaHelper: ITimplaHelper) => TaskFunction;
export interface IBabelLoaderOptions extends TransformOptions {
    /**
     * Default `false`. When set, the given directory will be used to cache the results of the loader. Future webpack builds will attempt to read from the cache to avoid needing to run the potentially expensive Babel recompilation process on each run. If the value is blank (loader: 'babel-loader?cacheDirectory') or true (loader: 'babel-loader?cacheDirectory=true'), the loader will use the default cache directory in node_modules/.cache/babel-loader or fallback to the default OS temporary file directory if no node_modules folder could be found in any root directory.
     */
    cacheDirectory?: boolean;
    /**
     * Default is a `string` composed by the babel-core's version, the babel-loader's version, the contents of the babel config file if it exists, and the value of the environment variable BABEL_ENV with a fallback to the NODE_ENV environment variable. This can be set to a custom value to force cache busting if the identifier changes.
     */
    cacheIdentifier?: string;
    /**
     * Default `true`. When set, each Babel transform output will be compressed with Gzip. If you want to opt-out of cache compression, set it to `false` -- your project may benefit from this if it transpiles thousands of files.
     */
    cacheCompression?: boolean;
    /**
     * Default `null`. The path of a module that exports a custom callback like the one that you'd pass to .custom(). Since you already have to make a new file to use this, it is recommended that you instead use .custom to create a wrapper loader. Only use this is you must continue using babel-loader directly, but still want to customize.
     */
    customize?: any;
}
/** ESLint loader options */
export interface IESLintLoaderOptions extends CLIEngine.Options {
    fix?: boolean;
    cache?: boolean;
    formatter?: boolean;
    eslintPath?: boolean;
    emitError?: boolean;
    emitWarning?: boolean;
    quiet?: boolean;
    failOnWarning?: boolean;
    failOnError?: boolean;
    outputReport?: boolean;
}
/** ForkTSChecker options */
export interface IForkTsCheckerOptions {
    tsconfig?: string;
    compilerOptions?: Partial<{}>;
    tslint?: string | true;
    tslintAutoFix?: boolean;
    watch?: string | string[];
    async?: boolean;
    ignoreDiagnostics?: number[];
    ignoreLints?: string[];
    reportFiles?: string[];
    colors?: boolean;
    logger?: Partial<{}>;
    formatter?: 'default' | 'codeframe' | ((message: any, useColors: boolean) => string);
    formatterOptions?: Partial<{}>;
    silent?: boolean;
    checkSyntacticErrors?: boolean;
    memoryLimit?: number;
    workers?: number;
    vue?: boolean;
}
export interface IHardSourceWebpackPluginOptions {
    cacheDirectory?: string;
    configHash?: (webpackConfig?: webpack.Configuration) => string;
    environmentHash?: {
        root: string;
        directories: string[];
        files: string[];
    };
    info?: {
        mode: 'none' | 'test';
        level: 'debug' | 'log' | 'info' | 'warn' | 'error';
    };
    cachePrune?: {
        maxAge: number;
        sizeThreshold: number;
    };
}
export interface IBaseHelper {
    /**
     * Resolves a given list of strings to the project `dest` directory.
     * @param string[] - a list of strings to merge with the `dest` path.
     * @return string - the resolved path.
     */
    projectDestPath: IProjectDestPath;
    /**
     * Resolves a given list of strings to the project `src` directory.
     * @param string[] - a list of strings to merge with the `src` path.
     * @return string - the resolved path.
     */
    projectSrcPath: IProjectSrcPath;
    /** The full timpla config. */
    timplaConfig: ITimplaConfig;
    /** Provides useful constants such as isDevelopment, isProduction and INIT_CWD */
    timplaProcess: ITimplaProcess;
}
export interface ITimplaWebpackHelper extends IBaseHelper {
    /** The generated webpack config, so far. */
    webpackConfig: webpack.Configuration;
    /** The webpack instance. */
    webpack: typeof webpack;
    /** A utility function to merge webpack config objects. See [Webpack Merge](https://www.npmjs.com/package/webpack-merge). */
    webpackMerge: IWebpackMerge;
}
export declare type IProjectSrcPath = (...paths: Array<string | undefined>) => string;
export declare type IProjectDestPath = (...paths: Array<string | undefined>) => string;
export interface ITimplaHelper extends IBaseHelper {
    /**  Access the browserSync instance. */
    browserSync: BrowserSyncInstance;
    /**  Access the gulp instance. You may register new tasks via `gulp.task`. */
    gulp: Gulp;
}
export interface IWebpackHotMiddlewareOptions {
    /** The path which the middleware is serving the event stream on */
    path?: boolean;
    /** Bundle name, specifically for multi-compiler mode */
    name?: boolean;
    /** The time to wait after a disconnection before attempting to reconnect */
    timeout?: boolean;
    /** Set to `false` to disable the DOM-based client-side overlay. */
    overlay?: boolean;
    /** Set to true to auto-reload the page when webpack gets stuck. */
    reload?: boolean;
    /** Set to true to disable informational console logging. */
    noInfo?: boolean;
    /** Set to true to disable all console logging. */
    quiet?: boolean;
    /** Set to true to use webpack publicPath as prefix of path. (We can set __webpack_public_path__ dynamically at runtime in the entry point, see note of output.publicPath) */
    dynamicPublicPath?: boolean;
    /** Set to `false` to use to prevent a connection being automatically opened from the client to the webpack back-endideal if you need to modify the options using the setOptionsAndConnect function */
    autoConnect?: boolean;
    /** An object to customize the client overlay colors as mentioned in the ansi-html package. */
    ansiColors?: boolean;
    /** An object to let you override or add new inline styles to the client overlay div. */
    overlayStyles?: boolean;
    /** Set to true to enable client overlay on warnings in addition to errors. */
    overlayWarnings?: boolean;
}
export interface IFullTimplaConfig {
    /** The base folder to include for compilation. Defaults to `./src`. */
    src: string;
    /** The base folder to output bundled files to. Defaults to `./dest`. */
    dest: string;
    staticFiles: false | {
        /** The folder to include for compilation. Defaults to `static`. */
        src: string;
        /** The folder to output bundled files to. Defaults to `./`. */
        dest: string;
        /** Options to pass to the gulp src task. This setting is ignored when an alternate task is provided. */
        srcOptions: SrcOptions;
        /** Options to pass to the gulp dest task. This setting is ignored when an alternate task is provided. */
        destOptions: DestOptions;
        /** A custom task to override the default with. */
        alternate?: ITimplaHofTask;
    };
    /**
     * Instructs timpla on how to run the clean task.
     * Set to `false` to disable the task.
     */
    clean: false | {
        /**
         * Extra files and folders to `clean` before Timpla runs the dev-server or full-build.
         * `Please take great care when specifying your own patterns`
         * Accepts an array of globs. Defaults to the project `dest` directory.
         */
        patterns?: string | ReadonlyArray<string>;
        /** Options to pass to the del function. */
        delOptions: Options;
    };
    /** Watch task options */
    watch: {
        /**
         * Gulp watch options.
         * Accepts gulp watch options for all tasks,
         * or a keyed-list of watch-options to apply to tasks individually.
         * - E.g. gulpWatch.svg = {} sets specific configs for the svg task.
         */
        gulpWatch: {
            [name: string]: any;
        };
    };
    /**
     * Instructs timpla on how to run the stylesheets task.
     * Set to `false` to disable the task.
     */
    stylesheets: false | {
        /** The folder to include for compilation. Defaults to `stylesheets`. */
        src: string;
        /** The folder to output bundled files to. Defaults to `stylesheets`. */
        dest: string;
        /** Options to pass to the gulp src task. This setting is ignored when an alternate task is provided. */
        srcOptions: SrcOptions;
        /** Options to pass to the gulp dest task. This setting is ignored when an alternate task is provided. */
        destOptions: DestOptions;
        /** [Cssnano](https://www.github.com/cssnano/cssnano) options */
        cssnanoOptions: CssNanoOptions;
        /** [Autoprefixer](https://github.com/postcss/autoprefixer#options) options */
        autoprefixerOptions: autoprefixer.Options;
        /** [PostCSS](https://github.com/postcss/postcss#options) options */
        postcssOptions: postcss.ProcessOptions;
        /** [Gulp/Node sass](https://www.npmjs.com/package/gulp-sass) options */
        sassOptions: INodeSassOptions;
        /** A list of extensions for BrowserSync to watch. Defaults to `['sass', 'scss', 'css']`. */
        extensions: string[];
        /** Development-specific stylesheets options */
        development: {
            /** Defaults to `true`. */
            sourceMap: boolean;
        };
        /** Production-specific stylesheets options */
        production: {
            /** Defaults to `false`. */
            sourceMap: boolean;
        };
        /** A custom task to override the default with. */
        alternate?: ITimplaHofTask;
    };
    /**
     * Instructs timpla on how to run the fonts task.
     * Set to `false` to disable the task.
     */
    fonts: false | {
        /** The folder to include for compilation. Defaults to `fonts`. */
        src: string;
        /** The folder to output bundled files to. Defaults to `fonts`. */
        dest: string;
        /** Options to pass to the gulp src task. This setting is ignored when an alternate task is provided. */
        srcOptions: SrcOptions;
        /** Options to pass to the gulp dest task. This setting is ignored when an alternate task is provided. */
        destOptions: DestOptions;
        /** A list of extensions for BrowserSync to watch. Defaults to `['woff2', 'woff', 'eot', 'ttf', 'svg']`. */
        extensions: string[];
        /** A custom task to override the default with. */
        alternate?: ITimplaHofTask;
    };
    /**
     * Instructs timpla on how to run the images task.
     * Set to `false` to disable the task.
     */
    images: false | {
        /** The folder to include for compilation. Defaults to `images`. */
        src: string;
        /** The folder to output bundled files to. Defaults to `images`. */
        dest: string;
        /** Options to pass to the gulp src task. This setting is ignored when an alternate task is provided. */
        srcOptions: SrcOptions;
        /** Options to pass to the gulp dest task. This setting is ignored when an alternate task is provided. */
        destOptions: DestOptions;
        /** A list of extensions for BrowserSync to watch. Defaults to `['jpg', 'png', 'svg', 'gif']`. */
        extensions: string[];
        /** A custom task to override the default with. */
        alternate?: ITimplaHofTask;
    };
    /**
     * Instructs timpla on how to run the svg task.
     * Set to `false` to disable the task.
     */
    svg: false | {
        /** Options for the gulp-svg plugin. */
        svgstore: IGulpSvgStore;
        /** The output filename for the concatanted svg file. Defaults to `icons.svg`. */
        outputName: string;
        /** The folder to include for compilation. Defaults to `svg`. */
        src: string;
        /** The folder to output bundled files to. Defaults to `images`. */
        dest: string;
        /** Options to pass to the gulp src task. This setting is ignored when an alternate task is provided. */
        srcOptions: SrcOptions;
        /** Options to pass to the gulp dest task. This setting is ignored when an alternate task is provided. */
        destOptions: DestOptions;
        /** A custom task to override the default with. */
        alternate?: ITimplaHofTask;
    };
    /**
     * Instructs timpla on how to run the html task.
     * Set to `false` to disable the task.
     */
    html: false | {
        /** The folder to include for compilation. Defaults to `html`. */
        src: string;
        /** The folder to output bundled files to. Defaults to ``. */
        dest: string;
        /** Options to pass to the gulp src task. This setting is ignored when an alternate task is provided. */
        srcOptions: SrcOptions;
        /** Options to pass to the gulp dest task. This setting is ignored when an alternate task is provided. */
        destOptions: DestOptions;
        /** A custom task to override the default with. */
        alternate?: ITimplaHofTask;
    };
    /**
     * Instructs timpla on how to run the javascripts task.
     */
    javascripts: {
        /** The folder to include for compilation. Defaults to `javascripts`. */
        src: string;
        /** The folder to output bundled files to. Defaults to `javascripts`. */
        dest: string;
        /** A list of extensions for BrowserSync to watch and Webpack to load. Defaults to `['js', 'jsx', 'ts', 'tsx']`. */
        extensions: string[];
        /**
         * The webpack entry config.
         * Accepts a Timpla-flavoured entry function or a webpack entry object.
         * Webpack is configured to use the projectRoot as the cwd.
         * Files you point in the entry config need to have the relative path to the javascripts folder.
         * Use the `resolve` helper to resolve files to the js src folder.
         */
        entry: ITimplaWebpackEntry;
        /**
         * Instructs webpack the base path for all the assets within your application.
         * Defaults to the `javascripts.dest` path. e.g. `/javascripts`
         */
        publicPath?: string;
        /**
         * Options for the babelLoader. Accepts babel-loader specific as well as Babel options.
         * For more advanced use-cases, e.g. manually specifying the babel config file to use.
         * Defaults to
         * - `cacheDirectory: true`
         * - `cacheCompression: false`
         * - `root: your project root`
         */
        babelLoaderOptions: IBabelLoaderOptions;
        /**
         * Accepts [webpackBundleAnalyzer](https://github.com/webpack-contrib/webpack-bundle-analyzer) options.
         * Set to `false` to disable webpackBundleAnalyzer.
         * Defaults to
         * - `statsFilename: timpla-webpack-stats.json`
         * - `openAnalyzer: true if Timpla hasn't fully reloaded.`
         */
        webpackBundleAnalyzerOptions: false | BundleAnalyzerPlugin.Options;
        /**
         * A customisation function that allows modifying the generated
         * webpack config before it is passed to webpack.
         */
        customizeWebpackConfig: ITimplaCustomizeWebpackConfig;
        /** Instructs timpla on how Webpack should run in dev-mode. */
        development: {
            /**
             * The webpack devtool to use.
             * Defaults to `cheap-module-eval-source-map`.
             */
            devtool: webpack.Options.Devtool;
            /**
             * Accepts [Webpack Hot Middleware](https://github.com/webpack-contrib/webpack-hot-middleware) options.
             * Set to `false` to disable [hot module reloading](https://webpack.js.org/concepts/hot-module-replacement/).
             * Defaults to
             * - `reload: true`
             * - `quiet: true`
             */
            webpackHotMiddlewareOptions: IWebpackHotMiddlewareOptions | false;
            /**
             * In dev-mode, Timpla supports ESLint via the ESLint loader.
             * By default, Timpla requires the eslint package to be installed.
             * - `npm i --save-dev eslint`
             * - Set to `false` to disable ESLint.
             */
            eslint: {
                esLintLoaderOptions: IESLintLoaderOptions;
            };
            /**
             * In dev-mode, Timpla uses Fork TS Checker, a webpack plugin that runs typescript export type checker on a separate process.
             * By default, Timpla requires dependent packages to be installed.
             * - `npm i --save-dev typescript tslint fork-ts-checker-webpack-plugin`
             * - Set to `false` to disable the plugin, and TSLint support.
             */
            tslint: {
                /** Accepts [Fork TS Checker](https://github.com/Realytics/fork-ts-checker-webpack-plugin) options. */
                forkTsCheckerOptions: IForkTsCheckerOptions;
            } | false;
        };
        /**
         * Instructs timpla on how to run Webpack in production.
         */
        production: {
            /**
             * The webpack devtool to use.
             * Defaults to `source-map`.
             */
            devtool: webpack.Options.Devtool;
            /**
             * Timpla uses Hard Source to improve production build times.
             * Set to `false` to disable the plugin.
             * Accepts [hard-source-webpack-plugin](https://github.com/mzgoddard/hard-source-webpack-plugin) options.
             */
            hardSourceOptions: false | IHardSourceWebpackPluginOptions;
            /**
             * [Terser Webpack Plugin](https://github.com/webpack-contrib/terser-webpack-plugin) options.
             * Timpla uses Terser to uglify the bundled JS files.
             * UglifyJS2 is outdated, and Terser is its successor.
             */
            terserConfig: Partial<{}>;
        };
    };
    /**
     * Configures the BrowserSync task.
     * Accepts [browser-sync](https://browsersync.io/docs/options) options.
     *
     * May also be used to individually `disable` the default
     * middleware added to proxy and server modes via middlewareConfig:
     * - webpackDevMiddleware - disableDev
     * - webpackHotMiddleware - disableHot
     * - historyApiFallback - disableSpa
     *
     * Set proxy/server.middleware to `false` to disable middleware.
     */
    browserSync: ITimplaBrowserSyncOptions;
    /** Instructs Timpla on after-build tasks. */
    production: {
        /**
         * Opens a specified target in an app after all the core-tasks run.
         * Uses the opn package to achieve this.
         * Defaults to `false`.
         */
        open: false | IOpenOptions;
        /**
         * Enables file-reving (hashed filenames).
         * Timpla revs your files and generates a json manifest file.
         * You'll need to write your own mapper to pick up the filenames from the rev manifest file.
         * Defaults to `false`.
         */
        rev: boolean;
    };
    /** Instructs Timpla on post-dev-launch tasks. */
    development: {
        /**
         * Opens a specified target in an app after all the core-tasks run.
         * Uses the opn package to achieve this.
         * Defaults to `false`.
         */
        open: false | IOpenOptions;
        /**
         * Extra files and folders for Timpla to watch.
         * When these change, Timpla reloads the dev-server.
         * Set to `false` to disable this option.
         * Defaults to `true`.
         */
        timplaWatch: boolean | string[];
    };
    /** Instructs Timpla on additional tasks */
    additionalTasks: {
        /** An init function that runs before all the other Timpla. */
        initialize?: (timplaHelper: ITimplaHelper) => any;
        /** Instructs Timpla to add extra `development tasks`. */
        development: {
            /** Adds an extra development `prebuild task`. Accepts a `higher-order-function`, i.e. returns a Gulp/Undertaker TaskFunction. */
            prebuild?: ITimplaHofTask;
            /** Adds an extra development `postbuild task`. Accepts a `higher-order-function`, i.e. returns a Gulp/Undertaker TaskFunction. */
            postbuild?: ITimplaHofTask;
        };
        /** Instructs Timpla to add extra `production tasks`. */
        production: {
            /** Adds an extra production `prebuild task`. Accepts a `higher-order-function`, i.e. returns a Gulp/Undertaker TaskFunction. */
            prebuild?: ITimplaHofTask;
            /** Adds an extra production `postbuild task`. Accepts a `higher-order-function`, i.e. returns a Gulp/Undertaker TaskFunction. */
            postbuild?: ITimplaHofTask;
        };
    };
}
/** The partialised Timpla Config so users' may opt in to only customise what they wish to. */
export declare type ITimplaConfig = IRecursivePartial<IFullTimplaConfig>;
